"""
This file contains the fields mapping for each endpoint and version related to Quick
Rules.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-apis
"""

from secrets_safe_library.constants.endpoints import GET_QUICK_RULES, GET_QUICK_RULES_ID
from secrets_safe_library.constants.versions import Version

fields = {
    GET_QUICK_RULES: {
        Version.DEFAULT: [
            "SmartRuleID",
            "OrganizationID",
            "Title",
            "Description",
            "Category",
            "Status",
            "LastProcessedDate",
            "IsReadOnly",
            "RuleType",
        ],
    },
    GET_QUICK_RULES_ID: {
        Version.DEFAULT: [
            "SmartRuleID",
            "OrganizationID",
            "Title",
            "Description",
            "Category",
            "Status",
            "LastProcessedDate",
            "IsReadOnly",
            "RuleType",
        ],
    },
}



"""Custom validator for Cerberus."""

import ipaddress
import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils


class CustomValidator(Validator):
    def _validate_is_required_if(self, is_required_if, field, value):
        """
        Custom rule to make a field required if another field has a specific value.

        The rule's arguments are validated against this schema:
        {'type': 'dict', 'schema': {'field': {'type': 'string'}, 'value': {}}}
        """
        other_field = is_required_if["field"]
        required_value = is_required_if["value"]

        # Check if the other field exists and has the required value
        if (
            other_field in self.document
            and self.document[other_field] == required_value
        ):
            if value is None or value == "":
                self._error(
                    field,
                    f"'{field}' is required when '{other_field}' is {required_value}",
                )

    def _validate_is_ip(self, is_ip, field, value):
        """Custom validation rule to check if a value is a valid IP address.

        The rule's arguments are validated against this schema:
        {'type': 'boolean'}
        """
        if is_ip and value:
            try:
                ipaddress.ip_address(value)  # Validate IP address
            except ValueError:
                self._error(field, f"'{value}' is not a valid IP address.")


class BaseValidator:
    """Base class for validators."""

    def validate(
        self,
        data: dict,
        schema: dict,
        allow_unknown: bool = False,
        update: bool = False,
    ) -> dict:
        """
        Validate data using the provided schema.

        Args:
            data (dict): Data to validate.
            schema (dict): Schema to validate against.
            allow_unknown (bool): Whether to allow unknown fields in the data.
            update (bool): Whether the validation is for an update operation.

        Raises:
            exceptions.OptionsError: If validation fails, an error is raised with
            details.

        Returns:
            dict: The validated data if validation is successful.
        """
        validator = CustomValidator(schema, allow_unknown=allow_unknown)

        if not validator.validate(data, update=update):
            utils.print_log(
                self._logger,
                f"Validation failed: {validator.errors}",
                logging.ERROR,
            )
            raise exceptions.OptionsError(f"Please check: {validator.errors}")

        return validator.document



"""Applications module, all the logic to manage applications from PS API"""

import logging

from secrets_safe_library import utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import GetByIdMixin, ListMixin


class Application(APIObject, GetByIdMixin, ListMixin):
    """
    Application class for managing applications via Password Safe API.

    This class provides functionality to:
    - Get a list of applications
      (requires Password Safe Account Management (Read) permission)
    - Get an application by ID
      (requires Password Safe Account Management (Read) permission)
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/applications")

    def get_managed_account_apps(self, account_id: int) -> list:
        """
        Returns a list of applications assigned to a managed account.

        Args:
            account_id (int): ID of the managed account.

        Returns:
            list: List of applications assigned to the managed account.
        """
        endpoint = f"/managedaccounts/{account_id}/applications"

        utils.print_log(
            self._logger,
            f"Calling get_managed_account_apps endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def assign_app_to_managed_account(
        self, account_id: int, application_id: int
    ) -> dict:
        """
        Assigns an application to a managed account.

        Args:
            account_id (int): ID of the managed account.
            application_id (int): ID of the application.

        Returns:
            dict: Response from the API.
        """
        endpoint = f"/managedaccounts/{account_id}/applications/{application_id}"

        utils.print_log(
            self._logger,
            f"Calling assign_app_to_managed_account endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, {}, include_api_version=False)

        return response.json()

    def remove_app_from_managed_account(
        self, account_id: int, application_id: int
    ) -> None:
        """
        Removes an application from a managed account.

        Args:
            account_id (int): ID of the managed account.
            application_id (int): ID of the application.

        Returns:
            None
        """
        endpoint = f"/managedaccounts/{account_id}/applications/{application_id}"

        utils.print_log(
            self._logger,
            f"Calling remove_app_from_managed_account endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint, expected_status_code=200)

    def unassign_all_apps_from_managed_account(self, account_id: int) -> None:
        """
        Unassigns all applications from a managed account.

        Args:
            account_id (int): ID of the managed account.

        Returns:
            None
        """
        endpoint = f"/managedaccounts/{account_id}/applications"

        utils.print_log(
            self._logger,
            f"Calling unassign_all_apps_from_managed_account endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint, expected_status_code=200)



"""Access levels module, all the logic to manage Access levels from BeyondInsight API"""

import logging

from secrets_safe_library import utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject


class AccessLevels(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/accesslevels")

    def get_access_levels(self) -> list:
        """
        Returns all access levels.

        API: GET AccessLevels

        Returns:
            list: List of access levels.
        """
        utils.print_log(
            self._logger,
            "Calling get_access_levels endpoint",
            logging.DEBUG,
        )
        response = self._run_get_request(self.endpoint, include_api_version=False)

        return response.json()

    def post_access_levels_usergroupid_smartruleid(
        self, usergroupid: int, smartruleid: int, accesslevelid: int
    ) -> None:
        """
        Assigns an access level to a user group and smart rule.

        API: POST UserGroups/{usergroupid}/SmartRules/{smartruleid}/AccessLevels

        Args:
            usergroupid (int): The user group ID.
            smartruleid (int): The smart rule ID.
            accesslevelid (int): The access level ID.

        Returns:
            dict: Response from the API.
        """

        endpoint = f"/usergroups/{usergroupid}/smartrules/{smartruleid}/accesslevels"

        payload = {"AccessLevelID": accesslevelid}

        utils.print_log(
            self._logger,
            f"Calling post_access_levels_usergroupid_smartruleid endpoint: {endpoint}",
            logging.DEBUG,
        )

        self._run_post_request(
            endpoint,
            payload,
            include_api_version=False,
            expected_status_code=200,
        )



"""SmartRules module, all the logic to manage Smart Rules from BeyondInsight API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_SMART_RULES_FILTER_ASSET_ATTRIBUTE,
)
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.smart_rules import fields as smart_rules_fields
from secrets_safe_library.mixins import (
    DeleteByIdMixin,
    DeleteByKeyMixin,
    GetByIdMixin,
    ListByKeyMixin,
    ListMixin,
)


class SmartRule(
    APIObject,
    DeleteByIdMixin,
    DeleteByKeyMixin,
    GetByIdMixin,
    ListMixin,
    ListByKeyMixin,
):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/smartrules")

        # Schema rules used for validations
        self._schema = {
            "title": {"type": "string", "maxlength": 75, "nullable": True},
            "rule_id": {"type": "integer", "nullable": True},
            "category": {"type": "string", "maxlength": 50, "nullable": True},
            "description": {"type": "string", "nullable": True},
            "process_immediately": {
                "type": "boolean",
                "default": True,
                "nullable": True,
            },
            "attribute_ids": {
                "type": "list",
                "schema": {"type": "integer"},
                "minlength": 1,
                "nullable": True,
            },
        }
        self._validator = Validator(self._schema)

    def list_assets_by_smart_rule_id(
        self, smart_rule_id: int, limit: int = None, offset: int = None
    ) -> list:
        """
        Returns a list of assets for the given smart rule ID.

        API: GET SmartRules/{id}/Assets

        Args:
            smart_rule_id (int): The Smart Rule ID.
            limit (int, optional): limit the results.
            offset (int, optional): skip the first (offset) number of assets.

        Returns:
            list: List of assets for the specified smart_rule_id.
        """

        attributes = {"rule_id": smart_rule_id}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        params = {
            "limit": limit,
            "offset": offset,
        }
        query_string = self.make_query_string(params)

        endpoint = f"/smartrules/{smart_rule_id}/assets?{query_string}"

        utils.print_log(
            self._logger,
            f"Calling list_assets_by_smart_rule_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)

        return response.json()

    def create_filter_asset_attribute(
        self,
        attribute_ids: list[int],
        title: str,
        category: str = "Smart Rules",
        description: str = "",
        process_immediately: bool = True,
    ) -> dict:
        """
        Creates a new Smart Rule with the attributes referenced by ID.

        API: POST SmartRules/FilterAssetAttribute

        Args:
            attribute_ids (list): List of attribute IDs to filter by.
            title (str): The title/name of the new Smart Rule, max 75 characters.
            category (str, optional): The category in which to place the Smart Rule,
                default 'Smart Rules', max string length is 50.
            description (str, optional): The Smart Rule description.
            process_immediately (bool, optional): True to process immediately, default
                True.

        Returns:
            dict: The created Smart Rule.
        """

        attributes = {
            "title": title,
            "category": category,
            "description": description,
            "attribute_ids": attribute_ids,
            "process_immediately": process_immediately,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            smart_rules_fields, POST_SMART_RULES_FILTER_ASSET_ATTRIBUTE, Version.DEFAULT
        )
        req_body = self.generate_request_body(req_structure, **attributes)

        endpoint = f"{self.endpoint}/filterassetattribute"

        utils.print_log(
            self._logger, f"Calling {endpoint} with payload: {req_body}", logging.DEBUG
        )
        response = self._run_post_request(
            endpoint, payload=req_body, include_api_version=False
        )

        return response.json()

    def list_smart_rules_by_user_group_id(self, user_group_id: int) -> list:
        """
        Returns a list of Smart Rules to which the given user group ID has at least
        read access.

        API: GET UserGroups/{id}/SmartRules

        Args:
            user_group_id (int): The User Group ID.

        Returns:
            list: List of Smart Rules for the specified user_group_id.
        """

        endpoint = f"/usergroups/{user_group_id}/smartrules"

        utils.print_log(
            self._logger,
            f"Calling list_smart_rules_by_user_group_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint)

        return response.json()

    def run_smart_rule(
        self, smart_rule_id: int, queue: bool = False
    ) -> tuple[dict, int]:
        """
        Processes a Smart Rule by ID.

        Response codes:
            200: Request successful. Smart Rule in the response body.
            409: Conflict: the Smart Rule is currently processing.

        API: POST SmartRules/{id}/Process

        Args:
            smart_rule_id (int): The Smart Rule ID.
            queue (bool, optional): True to queue the Smart Rule for processing,
                False to process it immediately. Defaults to False.

        Returns:
            tuple: A tuple containing the response JSON and the int status code.
        """

        endpoint = f"/smartrules/{smart_rule_id}/process?queue={str(queue).lower()}"

        utils.print_log(
            self._logger,
            f"Calling process_smart_rule_by_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, payload={}, expected_status_code=[200, 409]
        )
        status_code = response.status_code

        return response.json(), status_code



"""
This file contains the fields mapping for each endpoint and version related to
Requests.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/password-safe-api
"""

from secrets_safe_library.constants.endpoints import GET_REQUEST_SETS, POST_REQUEST_SETS
from secrets_safe_library.constants.versions import Version

fields = {
    GET_REQUEST_SETS: {
        Version.DEFAULT: [
            "RequestSetID",
            "Requests",
        ],
    },
    POST_REQUEST_SETS: {
        Version.DEFAULT: {
            "AccessTypes": list,
            "SystemID": int,
            "AccountID": int,
            "ApplicationID": int,
            "DurationMinutes": int,
            "Reason": str,
            "TicketSystemID": int,
            "TicketNumber": str,
        },
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
functional accounts.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_FUNCTIONAL_ACCOUNTS,
    GET_FUNCTIONAL_ACCOUNTS_ID,
    POST_FUNCTIONAL_ACCOUNTS,
)
from secrets_safe_library.constants.versions import Version

GET_FUNCTIONAL_ACCOUNTS_DEFAULT = [
    "FunctionalAccountID",
    "PlatformID",
    "DomainName",
    "AccountName",
    "DisplayName",
    "Description",
    "ElevationCommand",
    "SystemReferenceCount",
    "TenantID",
    "ObjectID",
    "AzureInstance",
]

fields = {
    GET_FUNCTIONAL_ACCOUNTS: {
        Version.DEFAULT: GET_FUNCTIONAL_ACCOUNTS_DEFAULT,
    },
    GET_FUNCTIONAL_ACCOUNTS_ID: {
        Version.DEFAULT: GET_FUNCTIONAL_ACCOUNTS_DEFAULT,
    },
    POST_FUNCTIONAL_ACCOUNTS: {
        Version.DEFAULT: {
            "PlatformID": int,
            "DomainName": str,
            "AccountName": str,
            "DisplayName": str,
            "Password": str,
            "PrivateKey": str,
            "Passphrase": str,
            "Description": str,
            "ElevationCommand": str,
            "TenantID": str,
            "ObjectID": str,
            "Secret": str,
            "ServiceAccountEmail": str,
            "AzureInstance": str,
        }
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
CREDENTIALS.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_CREDENTIALS_ALIASID,
    GET_CREDENTIALS_REQUESTID,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_CREDENTIALS_REQUESTID: {
        Version.DEFAULT: [
            "Credentials",
        ],
    },
    GET_CREDENTIALS_ALIASID: {
        Version.DEFAULT: [
            "AliasID",
            "AliasName",
            "SystemID",
            "SystemName",
            "AccountID",
            "AccountName",
            "DomainName",
            "Password",
            "PrivateKey",
            "Passphrase",
        ],
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
Permissions.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_PERMISSIONS,
    GET_USERGROUP_PERMISSIONS,
    POST_USERGROUP_PERMISSIONS,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_PERMISSIONS: {
        Version.DEFAULT: [
            "PermissionID",
            "Name",
        ],
    },
    GET_USERGROUP_PERMISSIONS: {
        Version.DEFAULT: [
            "PermissionID",
            "AccessLevelID",
        ],
    },
    POST_USERGROUP_PERMISSIONS: {
        Version.DEFAULT: {
            "PermissionID": int,
            "AccessLevelID": int,
        },
    },
}



"""Session Locking module, all the logic to lock sessions from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject


class SessionLocking(APIObject):
    """
    Class to interact with session locking in PS API.
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/sessions")

        # Schema rules used for validations
        self._schema = {
            "session_id": {"type": "integer", "nullable": True},
            "managed_account_id": {"type": "integer", "nullable": True},
            "managed_system_id": {"type": "integer", "nullable": True},
        }
        self._validator = Validator(self._schema)

    def post_session_lock_sessionid(
        self,
        session_id: int,
    ) -> None:
        """
        Locks a session by session ID.

        API: POST Sessions/{sessionid}/lock

        Args:
            session_id (int): The session ID.

        Returns:
            None
        """

        attributes = {
            "session_id": session_id,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"{self.endpoint}/{session_id}/lock"

        utils.print_log(
            self._logger,
            f"Calling post_session_lock_sessionid endpoint: {endpoint}",
            logging.DEBUG,
        )

        self._run_post_request(
            endpoint, payload={}, include_api_version=False, expected_status_code=204
        )

        utils.print_log(
            self._logger,
            f"Session with ID {session_id} locked successfully.",
            logging.INFO,
        )

    def post_session_lock_managed_account_id(
        self,
        managed_account_id: int,
    ) -> None:
        """
        Locks all active sessions by managed account ID.

        API: POST ManagedAccounts/{managedaccountid}/sessions/lock

        Args:
            managed_account_id (int): The managed account ID.

        Returns:
            None
        """

        attributes = {
            "managed_account_id": managed_account_id,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedaccounts/{managed_account_id}{self.endpoint}/lock"

        utils.print_log(
            self._logger,
            f"Calling post_session_lock_managed_account_id endpoint: {endpoint}",
            logging.DEBUG,
        )

        self._run_post_request(
            endpoint, payload={}, include_api_version=False, expected_status_code=204
        )

        utils.print_log(
            self._logger,
            f"Session with managed account ID {managed_account_id} "
            "locked successfully.",
            logging.INFO,
        )

    def post_session_lock_managed_system_id(
        self,
        managed_system_id: int,
    ) -> None:
        """
        Locks all active sessions by managed system ID.

        API: POST ManagedSystems/{managedsystemid}/sessions/lock

        Args:
            managed_system_id (int): The managed system ID.

        Returns:
            None
        """

        attributes = {
            "managed_system_id": managed_system_id,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedsystems/{managed_system_id}{self.endpoint}/lock"

        utils.print_log(
            self._logger,
            f"Calling post_session_lock_managed_system_id endpoint: {endpoint}",
            logging.DEBUG,
        )

        self._run_post_request(
            endpoint, payload={}, include_api_version=False, expected_status_code=204
        )

        utils.print_log(
            self._logger,
            f"Session with managed system ID {managed_system_id} locked successfully.",
            logging.INFO,
        )



"""Platforms module, all the logic to manage platforms from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import GetByIdMixin, ListMixin


class Platform(APIObject, ListMixin, GetByIdMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/platforms")

        self._schema = {
            "entity_type_id": {"type": "integer", "nullable": True},
        }
        self._validator = Validator(self._schema)

    def list_by_entity_type(self, entity_type_id: int) -> list:
        """
        Returns a list of Platforms by entity type ID.

        API: GET EntityTypes/{id}/Platforms

        Args:
            entity_type_id (int): The ID of the entity type.

        Returns:
            list: List of platforms associated with the specified entity type.
        """

        attributes = {"entity_type_id": entity_type_id}
        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/entitytypes/{entity_type_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            f"Calling list_by_entity_type endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()



"""Access Policies module, all the logic to read and test access policies from PS API"""

import logging
from typing import Tuple

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import ListMixin


class AccessPolicy(APIObject, ListMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/accesspolicies")

        # Schema rules used for validations
        self._schema = {
            "system_id": {"type": "integer", "min": 1, "required": True},
            "account_id": {"type": "integer", "min": 1, "required": True},
            "duration_minutes": {"type": "integer", "min": 1, "default": 60},
        }
        self._validator = Validator(self._schema)

    def test_access_policy(
        self,
        system_id: int,
        account_id: int,
        duration_minutes: int = 60,
    ) -> Tuple[dict, int]:
        """
        Tests access to a managed account and returns a list of Password Safe access
        policies that are available in the request window.

        API: POST AccessPolicies/Test

        Args:
            system_id (int): The ID of the system.
            account_id (int): The ID of the account.
            duration_minutes (int, optional): The duration in minutes for which the
                access is requested. Defaults to 60.

        Returns:
            Tuple[dict, int]: Tuple containing the JSON response and the HTTP status
                code.
        """
        attributes = {
            "system_id": system_id,
            "account_id": account_id,
            "duration_minutes": duration_minutes,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "SystemId": system_id,
            "AccountId": account_id,
            "DurationMinutes": duration_minutes,
        }
        endpoint = f"{self.endpoint}/test"

        utils.print_log(
            self._logger,
            f"Calling test access policy endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, payload, include_api_version=False, expected_status_code=200
        )

        return response.json(), response.status_code



"""TicketSystems module, all the logic to manage ticket systems from PS API"""

import logging

from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import ListMixin


class TicketSystems(APIObject, ListMixin):
    """
    Class to manage Ticket Systems from Password Safe API.

    This class provides methods to list registered ticket systems.
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/ticketsystems")



import logging
import os

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from secrets_safe_library import authentication, cert_util, secrets_safe, utils

env = os.environ

CLIENT_ID = env["CLIENT_ID"] if "CLIENT_ID" in env else None
CLIENT_SECRET = env["CLIENT_SECRET"] if "CLIENT_SECRET" in env else None
API_URL = env["API_URL"] if "API_URL" in env else None
VERIFY_CA = (
    False if "VERIFY_CA" in env and env["VERIFY_CA"].lower() == "false" else True
)
SECRET = env["SECRET"].strip() if "SECRET" in env else None
SECRET_LIST = (
    env["SECRET_LIST"].strip().split(",")
    if "SECRET_LIST" in env and env["SECRET_LIST"].strip() != ""
    else None
)

API_KEY = None

LOGGER_NAME = "custom_logger"

# the recommended version is 3.1. If no version is specified,
# the default API version 3.0 will be used
RECOMMENDED_API_VERSION = "3.1"

logging.basicConfig(
    format="%(asctime)-5s %(name)-15s %(levelname)-8s %(message)s", level=logging.DEBUG
)

logger = logging.getLogger(LOGGER_NAME)

CERTIFICATE_PATH = None
if "CERTIFICATE_PATH" in env and len(env["CERTIFICATE_PATH"]) > 0:
    CERTIFICATE_PATH = env["CERTIFICATE_PATH"]

CERTIFICATE_PASSWORD = (
    env["CERTIFICATE_PASSWORD"]
    if "CERTIFICATE_PASSWORD" in env and CERTIFICATE_PATH
    else ""
)

TIMEOUT_CONNECTION_SECONDS = 30
TIMEOUT_REQUEST_SECONDS = 30


def main():
    try:

        decrypt_obj = cert_util.CertUtil(CERTIFICATE_PATH, CERTIFICATE_PASSWORD, logger)

        certificate = decrypt_obj.get_certificate()
        certificate_key = decrypt_obj.get_certificate_key()

        with requests.Session() as session:
            retry_strategy = Retry(
                total=3,
                backoff_factor=0.2,
                status_forcelist=[400, 408, 500, 502, 503, 504],
                allowed_methods=["GET", "POST"],
            )
            adapter = HTTPAdapter(max_retries=retry_strategy)
            session.mount("https://", adapter)
            session.mount("http://", adapter)

            certificate, certificate_key = utils.prepare_certificate_info(
                certificate, certificate_key
            )

            authentication_obj = authentication.Authentication(
                req=session,
                timeout_connection=TIMEOUT_CONNECTION_SECONDS,
                timeout_request=TIMEOUT_REQUEST_SECONDS,
                api_url=API_URL,
                client_id=CLIENT_ID,
                client_secret=CLIENT_SECRET,
                certificate=certificate,
                certificate_key=certificate_key,
                verify_ca=VERIFY_CA,
                logger=logger,
                api_version=RECOMMENDED_API_VERSION,
            )

            get_api_access_response = authentication_obj.get_api_access()

            if get_api_access_response.status_code != 200:
                utils.print_log(
                    logger,
                    f"Please check credentials, error {get_api_access_response.text}",
                    logging.ERROR,
                )
                return

            if not SECRET and not SECRET_LIST:
                utils.print_log(
                    logger,
                    "Nothing to do, SECRET and SECRET_LIST parameters are empty!",
                    logging.ERROR,
                )
                return

            secrets_safe_obj = secrets_safe.SecretsSafe(
                authentication=authentication_obj, logger=logger, separator="/"
            )

            if SECRET:
                get_secret_response = secrets_safe_obj.get_secret(SECRET)
                utils.print_log(
                    logger, f"=> Retrive secret: {get_secret_response}", logging.DEBUG
                )
            if SECRET_LIST:
                get_secrets_response = secrets_safe_obj.get_secrets(SECRET_LIST)
                utils.print_log(
                    logger, f"=> Retrive secrets: {get_secrets_response}", logging.DEBUG
                )

            # get all secrets inside of a specific folder, using folder path.
            folder_path = "oauthgrp/folder1/test"
            get_secret_response = secrets_safe_obj.get_all_secrets_by_folder_path(
                folder_path
            )
            utils.print_log(
                logger,
                f"=> Secrets in {folder_path}: {get_secret_response}",
                logging.DEBUG,
            )

            authentication_obj.sign_app_out()

        del session

    except Exception as e:
        utils.print_log(logger, e, logging.ERROR)


# calling main method
main()



"""
This file contains the fields mapping for each endpoint and version related to Smart
Rules.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-apis
- https://docs.beyondtrust.com/bips/docs/passwordsafe-apis
- https://docs.beyondtrust.com/bips/docs/secrets-safe-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_SMART_RULES,
    GET_SMART_RULES_ID,
    POST_SMART_RULES_FILTER_ASSET_ATTRIBUTE,
)
from secrets_safe_library.constants.versions import Version

GET_SMART_RULES_DEFAULT = [
    "SmartRuleID",
    "OrganizationIDs",
    "Title",
    "Description",
    "Category",
    "Status",
    "LastProcessedDate",
    "IsReadOnly",
    "RuleType",
]

fields = {
    GET_SMART_RULES: {Version.DEFAULT: GET_SMART_RULES_DEFAULT},
    GET_SMART_RULES_ID: {
        Version.DEFAULT: GET_SMART_RULES_DEFAULT,
    },
    POST_SMART_RULES_FILTER_ASSET_ATTRIBUTE: {
        Version.DEFAULT: {
            "AttributeIDs": list,
            "Title": str,
            "Category": str,
            "Description": str,
            "ProcessImmediately": bool,
        }
    },
}



"""
This file contains the fields mapping for each endpoint and version related to ISA
Requests.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
"""

from secrets_safe_library.constants.endpoints import POST_ISA_REQUESTS
from secrets_safe_library.constants.versions import Version

fields = {
    POST_ISA_REQUESTS: {
        Version.DEFAULT.value: {
            "SystemID": int,
            "AccountID": int,
            "DurationMinutes": int,
            "Reason": str,
        },
    }
}



"""
This file contains the fields mapping for each endpoint and version related to
Entitlements.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-apis
"""

from secrets_safe_library.constants.endpoints import GET_ENTITLEMENTS
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ENTITLEMENTS: {
        Version.DEFAULT: [
            "GroupID",
            "Name",
            "SmartRuleId",
            "Title",
            "SmartRuleType",
            "AccessLevel",
            "RoleId",
            "RoleName",
            "DedicatedAccountPermissionOverride",
            "DedicatedToAppUserID",
            "DedicatedToAppUserName",
            "IsAdministratorGroup",
            "UserID",
            "UserName",
            "ManagedAccountId",
            "AccountName",
            "RationalizedSystemName",
            "ApplicationName",
            "AccessPolicyName",
            "ManagedSystemID",
            "ApplicationID",
        ],
    },
}



"""
Mixin classes for Secrets Safe Library.
Optional classes to extend APIObject functionality.

These mixins need to be used with APIObject class.
"""

import logging

from secrets_safe_library import utils


class ListMixin:
    """
    Mixin class to add list functionality.
    """

    def list(self) -> list:
        """
        Generic listing function.

        Returns:
            list: List of elements for defined endpoint.
        """

        utils.print_log(
            self._logger, f"Calling list endpoint: {self.endpoint}", logging.DEBUG
        )
        response = self._run_get_request(self.endpoint)

        return response.json()


class GetByIdMixin:
    """
    Mixin class to add get by ID functionality.
    """

    def get_by_id(self, object_id: str | int, include_api_version=False) -> dict:
        """
        Generic method to get an object by ID.

        Args:
            object_id (str | int): The ID of the object to retrieve.
            include_api_version (bool, optional): Whether to include the API version
                in the request. Defaults to False.

        Returns:
            dict: The object retrieved by ID.
        """
        endpoint = f"{self.endpoint}/{object_id}"

        utils.print_log(
            self._logger, f"Calling get_by_id endpoint: {endpoint}", logging.DEBUG
        )
        response = self._run_get_request(
            endpoint, include_api_version=include_api_version
        )

        return response.json()


class ListByKeyMixin:
    """
    Mixin class to add list/retrieve by key functionality.
    """

    def list_by_key(self, key: str, value: str) -> list | dict:
        """
        Generic method to list objects by a specific key and value.

        Args:
            key (str): The key to filter by.
            value (str): The value to filter by.

        Returns:
            list | dict: List of objects or object matching the key-value pair query
                string.
        """
        endpoint = f"{self.endpoint}?{key}={value}"

        utils.print_log(
            self._logger, f"Calling list_by_key endpoint: {endpoint}", logging.DEBUG
        )
        response = self._run_get_request(endpoint)

        return response.json()


class DeleteByIdMixin:
    """
    Mixin class to add delete by ID functionality.
    """

    def delete_by_id(
        self, object_id: str | int, expected_status_code: int = 204
    ) -> None:
        """
        Generic method to delete an object by ID.

        Args:
            object_id (str | int): The ID of the object to delete.
            expected_status_code (int, optional): The expected status code for a
                successful deletion. Defaults to 204.

        Raises:
            exceptions.DeletionError: If the deletion fails.
        """
        endpoint = f"{self.endpoint}/{object_id}"

        utils.print_log(
            self._logger, f"Calling delete_by_id endpoint: {endpoint}", logging.DEBUG
        )
        self._run_delete_request(endpoint, expected_status_code=expected_status_code)


class DeleteByKeyMixin:
    """
    Mixin class to add delete by query parameter functionality.
    i.e.: DELETE QuickRules?title={title}, where key is "title" and value is the value
    to delete.
    """

    def delete_by_key(
        self, key: str, value: str, expected_status_code: int = 204
    ) -> None:
        """
        Generic method to delete objects by a specific key and value.

        Args:
            key (str): The key to delete by.
            value (str): The value to delete by.
            expected_status_code (int, optional): The expected status code for a
                successful deletion. Defaults to 204.

        Returns:
            None.
        """
        endpoint = f"{self.endpoint}?{key}={value}"

        utils.print_log(
            self._logger, f"Calling delete_by_key endpoint: {endpoint}", logging.DEBUG
        )
        _ = self._run_delete_request(
            endpoint, expected_status_code=expected_status_code
        )



"""
This file contains the fields mapping for each endpoint and version related to
DATABASES.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-api
- https://docs.beyondtrust.com/bips/docs/secrets-safe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_DATABASES,
    GET_DATABASES_ASSET_ID,
    GET_DATABASES_ID,
    POST_DATABASES_ASSET_ID,
    PUT_DATABASES_ID,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_DATABASES: {
        Version.DEFAULT: [
            "AssetID",
            "DatabaseID",
            "PlatformID",
            "InstanceName",
            "IsDefaultInstance",
            "Port",
            "Version",
            "Template",
        ],
    },
    GET_DATABASES_ID: {
        Version.DEFAULT: [
            "AssetID",
            "DatabaseID",
            "PlatformID",
            "InstanceName",
            "IsDefaultInstance",
            "Port",
            "Version",
        ],
    },
    GET_DATABASES_ASSET_ID: {
        Version.DEFAULT: [
            "AssetID",
            "DatabaseID",
            "PlatformID",
            "InstanceName",
            "IsDefaultInstance",
            "Port",
            "Version",
            "Template",
        ],
    },
    POST_DATABASES_ASSET_ID: {
        Version.DEFAULT: {
            "PlatformID": int,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Port": int,
            "Version": str,
            "Template": str,
        },
    },
    PUT_DATABASES_ID: {
        Version.DEFAULT: {
            "PlatformID": int,
            "InstanceName": str,
            "IsDefaultInstance": bool,
            "Port": int,
            "Version": str,
            "Template": str,
        },
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
Subscription Deliveries.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-apis
"""

from secrets_safe_library.constants.endpoints import (
    POST_SUBSCRIPTIONS_DELIVERY_DOWNLOAD,
)
from secrets_safe_library.constants.versions import Version

fields = {
    POST_SUBSCRIPTIONS_DELIVERY_DOWNLOAD: {
        Version.DEFAULT: [
            "ReportDeliveryId",
            "ScheduleId",
            "Filename",
            "ApplicationType",
            "Snapshot",
        ]
    },
}



"""Requests module, all the logic to manage requests from Password Safe API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_REQUESTS,
    POST_REQUESTS_ALIASES,
)
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.requests import fields as requests_fields


class Request(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger)

        # Schema rules used for validations
        self._schema = {
            "access_type": {
                "type": "string",
                "allowed": ["View", "RDP", "SSH", "App"],
                "nullable": True,
            },
            "duration_minutes": {
                "type": "integer",
                "min": 1,
                "max": 525600,
                "nullable": True,
            },
            "reason": {
                "type": "string",
                "maxlength": 1000,
                "nullable": True,
            },
        }
        self._validator = Validator(self._schema)

    def get_requests(self, status: str = None, queue: str = None) -> list:
        """
        Returns all Requests.

        API: GET Requests

        Args:
            status (str, optional): Status of requests to return. Options:
                    - all (default): Both active and pending requests.
                    - pending:  Requests that have not yet been approved.
                    - active: Requests approved (including auto-approved).
            queue (str, optional): Type of request queue to return. Options:
                    - req (default): Requestor queue, returns requests available to the
                                     user as a requestor.
                    - app: Approver queue, returns requests for an approver or
                           requestor/approver that have either been approved by the user
                           (active) or have not yet been approved (pending).

        Returns:
            list: List of Requests.
        """
        params = {"status": status, "queue": queue}

        query_string = self.make_query_string(params)
        endpoint = f"/requests?{query_string}"

        utils.print_log(
            self._logger,
            f"Calling get_requests endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_request(
        self,
        system_id: int,
        account_id: int,
        duration_minutes: int,
        application_id: int = None,
        reason: str = None,
        access_type: str = None,
        access_policy_schedule_id: int = None,
        conflict_option: str = None,
        ticket_system_id: int = None,
        ticket_number: str = None,
        rotate_on_checkin: bool = False,
    ) -> dict:
        """
        Creates a new Request.

        API: POST Requests

        Args:
            system_id (int): ID of the managed system to request.
            account_id (int): ID of the managed account to request.
            duration_minutes (int): The request duration (in minutes).
            application_id (int, optional): The Application ID.
                                            Required when AccessType=App
            reason (str, optional): Reason for the request.
            access_type (str, optional): Type of access requested.
                                         Options: View, RDP, SSH, App.
            access_policy_schedule_id (int, optional): Access Policy Schedule ID.
            conflict_option (str, optional): Conflict option for the request.
            ticket_system_id (int, optional): Ticket System ID.
            ticket_number (str, optional): Ticket number associated with the request.
            rotate_on_checkin (bool, optional): Whether to rotate the password
                                                 on check-in/expiry.
        Returns:
            dict: The created Request.
        """
        attributes = {
            "access_type": access_type,
            "duration_minutes": duration_minutes,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            requests_fields, POST_REQUESTS, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            system_id=system_id,
            account_id=account_id,
            duration_minutes=duration_minutes,
            application_id=application_id,
            reason=reason,
            access_type=access_type,
            access_policy_schedule_id=access_policy_schedule_id,
            conflict_option=conflict_option,
            ticket_system_id=ticket_system_id,
            ticket_number=ticket_number,
            rotate_on_checkin=rotate_on_checkin,
        )

        endpoint = "/requests"
        utils.print_log(
            self._logger,
            f"Calling post_request endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def post_request_alias(
        self,
        alias_id: int,
        duration_minutes: int,
        access_type: str = None,
        reason: str = None,
        access_policy_schedule_id: int = None,
        conflict_option: str = None,
        ticket_system_id: int = None,
        ticket_number: str = None,
        rotate_on_checkin: bool = False,
    ) -> dict:
        """
        Creates a new release request using an alias.

        API: POST Aliases/{aliasId}/Requests

        Args:
            alias_id (int): ID of the managed account alias.
            duration_minutes (int): The Request duration (in minutes).
            access_type (str, optional): Type of access requested.
                                         Options: View, RDP, SSH, App.
            reason (str, optional): Reason for the request.
            access_policy_schedule_id (int, optional): Access Policy Schedule ID.
            conflict_option (str, optional): Conflict option for the request.
            ticket_system_id (int, optional): Ticket System ID.
            ticket_number (str, optional): Ticket number associated with the request.
            rotate_on_checkin (bool, optional): Whether to rotate the password
                                                 on check-in/expiry.
        Returns:
            dict: The created Request.
        """
        attributes = {
            "duration_minutes": duration_minutes,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            requests_fields, POST_REQUESTS_ALIASES, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            access_type=access_type,
            reason=reason,
            access_policy_schedule_id=access_policy_schedule_id,
            conflict_option=conflict_option,
            ticket_system_id=ticket_system_id,
            ticket_number=ticket_number,
            rotate_on_checkin=rotate_on_checkin,
        )

        endpoint = f"/aliases/{alias_id}/requests"
        utils.print_log(
            self._logger,
            f"Calling post_request endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def put_request_checkin(
        self,
        request_id: int,
        reason: str = None,
    ) -> None:
        """
        Checks-in/releases a request before it has expired.

        API: PUT Requests/{id}/checkin

        Args:
            request_id (int): The Request ID.
            reason (str, optional): Reason for the check-in.
        """

        attibute = {
            "reason": reason,
        }
        if not self._validator.validate(attibute, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Reason": reason,
        }

        endpoint = f"/requests/{request_id}/checkin"
        utils.print_log(
            self._logger,
            f"Calling put_request_checkin endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_put_request(endpoint, payload, include_api_version=False)

    def put_request_approve(
        self,
        request_id: int,
        reason: str = None,
    ) -> None:
        """
        Approves a pending request.

        API: PUT Requests/{id}/approve

        Args:
            request_id (int): The Request ID.
            reason (str, optional): Reason for the approval.
        """

        attibute = {
            "reason": reason,
        }
        if not self._validator.validate(attibute, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Reason": reason,
        }

        endpoint = f"/requests/{request_id}/approve"
        utils.print_log(
            self._logger,
            f"Calling put_request_approve endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_put_request(endpoint, payload, include_api_version=False)

    def put_request_deny(
        self,
        request_id: int,
        reason: str = None,
    ) -> None:
        """
        Denies/cancels an active or pending request.

        API: PUT Requests/{id}/deny

        Args:
            request_id (int): The Request ID.
            reason (str, optional): Reason for the denial.
        """

        attibute = {
            "reason": reason,
        }
        if not self._validator.validate(attibute, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Reason": reason,
        }

        endpoint = f"/requests/{request_id}/deny"
        utils.print_log(
            self._logger,
            f"Calling put_request_deny endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_put_request(endpoint, payload, include_api_version=False)

    def put_request_rotate_on_checkin(
        self,
        request_id: int,
    ) -> None:
        """
        Updates a request to rotate the credentials on check-in/expiry.

        API: PUT Requests/{id}/rotateoncheckin

        Args:
            request_id (int): The Request ID.

        Returns:
            None
        """

        endpoint = f"/requests/{request_id}/rotateoncheckin"
        utils.print_log(
            self._logger,
            f"Calling put_requests_rotate_on_checkin endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_put_request(endpoint, payload={}, include_api_version=False)



"""
This file contains the fields mapping for each endpoint and version related to Access
Policies.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_ACCESS_POLICIES,
    POST_ACCESS_POLICIES_TEST,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_ACCESS_POLICIES: {
        Version.DEFAULT: [
            "AccessPolicyID",
            "Name:string",
            "Description",
            "Schedules",
        ],
    },
    POST_ACCESS_POLICIES_TEST: {
        Version.DEFAULT: [
            "AccessPolicyID",
            "Name:string",
            "Description",
            "Schedules",
        ],
    },
}



"""ISA Requests module, all the logic to manage ISA Requests from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject


class ISARequest(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/isarequests")

        self._schema = {
            "system_id": {"type": "integer", "nullable": False},
            "reason": {"type": "string", "maxlength": 256, "nullable": True},
            "account_id": {"type": "integer", "nullable": False},
            "duration_minutes": {"type": "integer", "nullable": True},
        }
        self._validator = Validator(self._schema)

    def create_isa_request(
        self,
        system_id: int,
        account_id: int,
        duration_minutes: int = None,
        reason: str = None,
    ) -> dict:
        """
        Creates a new Information Systems Administrator (ISA) release request.

        API: POST ISARequests

        Args:
            system_id (int): ID of the managed system to request.
            account_id (int): ID of the managed account to request.
            duration_minutes (int, optional): The request duration in minutes. Defaults
                to None.
            reason (str, optional): The reason for the request. Defaults to None.

        Returns:
            dict: Response containing requested credentials.
        """

        attributes = {
            "system_id": system_id,
            "reason": reason,
            "account_id": account_id,
            "duration_minutes": duration_minutes,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "SystemID": system_id,
            "AccountID": account_id,
            "DurationMinutes": duration_minutes,
            "Reason": reason,
        }

        utils.print_log(
            self._logger,
            f"Calling create_isa_request endpoint: {self.endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            self.endpoint, payload, include_api_version=False
        )

        return response.json()



"""
This file contains the fields mapping for each endpoint and version related to
Keystrokes.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_KEYSTROKES_ID,
    GET_SESSIONS_SESSIONID_KEYSTROKES,
    POST_KEYSTROKES_SEARCH,
)
from secrets_safe_library.constants.versions import Version

GET_KEYSTROKES_DEFAULT = [
    "KeystrokeID",
    "SessionID",
    "TimeMarker",
    "Type",
    "Data",
]

fields = {
    GET_SESSIONS_SESSIONID_KEYSTROKES: {Version.DEFAULT: GET_KEYSTROKES_DEFAULT},
    GET_KEYSTROKES_ID: {
        Version.DEFAULT: GET_KEYSTROKES_DEFAULT,
    },
    POST_KEYSTROKES_SEARCH: {
        Version.DEFAULT: {
            "Data": str,
            "Type": int,
        }
    },
}



"""
This file contains the fields mapping for each endpoint and version related to
EPM Policies.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/beyondinsight-apis
"""

from secrets_safe_library.constants.endpoints import (
    POST_EPM_POLICIES_ID_EPMAPPLICATIONS_ADD,
)
from secrets_safe_library.constants.versions import Version

fields = {
    POST_EPM_POLICIES_ID_EPMAPPLICATIONS_ADD: {
        Version.DEFAULT: {
            "GroupName": str,
            "Name": str,
            "Path": str,
            "Publisher": str,
            "ChildrenInheritToken": bool,
        }
    },
}



"""
Attributes module, all the logic to manage attributes from BeyondInsight and
Password Safe API
"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_ATTRIBUTE_ATTRIBUTE_TYPE_ID
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.attributes import fields as attributes_fields
from secrets_safe_library.mixins import DeleteByIdMixin, GetByIdMixin


class Attributes(APIObject, DeleteByIdMixin, GetByIdMixin):
    """
    Attributes class to manage attributes in BeyondInsight and Password Safe API
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/attributes")

        # Schema rules used for validations
        self._schema = {
            "attribute_type_id": {"type": "integer", "nullable": False},
            "managed_account_id": {"type": "integer", "nullable": False},
            "managed_system_id": {"type": "integer", "nullable": False},
            "attribute_id": {"type": "integer", "nullable": False},
            "short_name": {
                "type": "string",
                "minlength": 1,
                "maxlength": 64,
                "nullable": False,
            },
            "long_name": {
                "type": "string",
                "minlength": 1,
                "maxlength": 64,
                "nullable": False,
            },
            "description": {
                "type": "string",
                "minlength": 1,
                "maxlength": 255,
                "nullable": False,
            },
        }
        self._validator = Validator(self._schema)

    def get_attributes_by_attribute_type_id(self, attribute_type_id: int) -> dict:
        """
        Returns a list of attribute definitions by attribute type.

        API: GET AttributeTypes/{attributeTypeID}/Attributes

        Args:
            attribute_type_id (int): The ID of the attribute type.

        Returns:
            dict: Attributes associated with the specified attribute type.
        """

        attributes = {"attribute_type_id": attribute_type_id}

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/attributetypes/{attribute_type_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            f"Calling get attributes by attribute type ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_attribute_by_attribute_type_id(
        self,
        attribute_type_id: int,
        short_name: str,
        long_name: str,
        description: str,
        parent_attribute_id: int = None,
        value_int: int = None,
    ) -> dict:
        """
        Creates a new attribute definition by attribute type ID.

        API: POST AttributeTypes/{attributeTypeID}/Attributes

        Args:
            attribute_type_id (int): The ID of the attribute type.
            short_name (str): The short name of the attribute.
            long_name (str): The long name of the attribute.
            description (str): The description of the attribute.
            parent_attribute_id (int, optional): The ID of the parent attribute.
            value_int (int, optional): The integer value of the attribute.

        Returns:
            dict: The created attribute.
        """

        attributes = {
            "attribute_type_id": attribute_type_id,
            "short_name": short_name,
            "long_name": long_name,
            "description": description,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            attributes_fields, POST_ATTRIBUTE_ATTRIBUTE_TYPE_ID, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            short_name=short_name,
            long_name=long_name,
            description=description,
            parent_attribute_id=parent_attribute_id,
            value_int=value_int,
        )

        endpoint = f"/attributetypes/{attribute_type_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            f"Calling post attribute by attribute type ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()

    def get_attributes_by_managed_account_id(self, managed_account_id: int) -> dict:
        """
        Returns a list of attribute definitions by managed account ID.

        API: GET ManagedAccounts/{managedAccountID}/Attributes

        Args:
            managed_account_id (int): The ID of the managed account.

        Returns:
            dict: Attributes associated with the specified managed account.
        """

        attributes = {"managed_account_id": managed_account_id}

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedaccounts/{managed_account_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            f"Calling get attributes by managed account ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(
            endpoint, include_api_version=False, expected_status_code=[200, 201]
        )

        return response.json()

    def get_attributes_by_managed_system_id(self, managed_system_id: int) -> dict:
        """
        Returns a list of attribute definitions by managed system ID.

        API: GET ManagedSystems/{managedSystemID}/Attributes

        Args:
            managed_system_id (int): The ID of the managed system.

        Returns:
            dict: Attributes associated with the specified managed system.
        """

        attributes = {"managed_system_id": managed_system_id}

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedsystems/{managed_system_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            f"Calling get attributes by managed system ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_attribute_by_managed_account_id(
        self, managed_account_id: int, attribute_id: int
    ) -> dict:
        """
        Assigns an attribute to a managed account.

        API: POST ManagedAccounts/{managedAccountID}/Attributes/{attributeID}

        Args:
            managed_account_id (int): The ID of the managed account.
            attribute_id (int): The ID of the attribute.

        Returns:
            dict: The created attribute.
        """

        attributes = {
            "managed_account_id": managed_account_id,
            "attribute_id": attribute_id,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = (
            f"/managedaccounts/{managed_account_id}{self.endpoint}/{attribute_id}"
        )

        utils.print_log(
            self._logger,
            f"Calling post attribute by managed account ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, payload={}, include_api_version=False
        )

        return response.json()

    def post_attribute_by_managed_system_id(
        self, managed_system_id: int, attribute_id: int
    ) -> dict:
        """
        Assigns an attribute to a managed system.

        API: POST ManagedSystems/{managedSystemID}/Attributes/{attributeID}

        Args:
            managed_system_id (int): The ID of the managed system.
            attribute_id (int): The ID of the attribute.

        Returns:
            dict: The created attribute.
        """

        attributes = {
            "managed_system_id": managed_system_id,
            "attribute_id": attribute_id,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedsystems/{managed_system_id}{self.endpoint}/{attribute_id}"

        utils.print_log(
            self._logger,
            f"Calling post attribute by managed system ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(
            endpoint, payload={}, include_api_version=False
        )

        return response.json()

    def delete_attributes_by_managed_account_id(self, managed_account_id: int) -> None:
        """
        Deletes all managed account attributes by managed account ID.

        API: DELETE ManagedAccounts/{managedAccountID}/Attributes

        Args:
            managed_account_id (int): The ID of the managed account.
        """

        attributes = {"managed_account_id": managed_account_id}

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedaccounts/{managed_account_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            f"Calling delete attributes by managed account ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

    def delete_attributes_by_managed_account_id_attribute_id(
        self, managed_account_id: int, attribute_id: int
    ) -> None:
        """
        Deletes a managed account attribute by managed account ID and attribute ID.

        API: DELETE ManagedAccounts/{managedAccountID}/Attributes/{attributeID}

        Args:
            managed_account_id (int): The ID of the managed account.
            attribute_id (int): The ID of the attribute.
        """

        attributes = {
            "managed_account_id": managed_account_id,
            "attribute_id": attribute_id,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = (
            f"/managedaccounts/{managed_account_id}{self.endpoint}/{attribute_id}"
        )

        utils.print_log(
            self._logger,
            "Calling delete attributes by managed account ID"
            f" and attribute ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

    def delete_attributes_by_managed_system_id(self, managed_system_id: int) -> None:
        """
        Deletes all managed system attributes by managed system ID.

        API: DELETE ManagedSystems/{managedSystemID}/Attributes

        Args:
            managed_system_id (int): The ID of the managed system.
        """

        attributes = {"managed_system_id": managed_system_id}

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedsystems/{managed_system_id}{self.endpoint}"

        utils.print_log(
            self._logger,
            f"Calling delete attributes by managed system ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

    def delete_attributes_by_managed_system_id_attribute_id(
        self, managed_system_id: int, attribute_id: int
    ) -> None:
        """
        Deletes a managed system attribute by managed system ID and attribute ID.

        API: DELETE ManagedSystems/{managedSystemID}/Attributes/{attributeID}

        Args:
            managed_system_id (int): The ID of the managed system.
            attribute_id (int): The ID of the attribute.
        """

        attributes = {
            "managed_system_id": managed_system_id,
            "attribute_id": attribute_id,
        }

        if not self._validator.validate(attributes):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        endpoint = f"/managedsystems/{managed_system_id}{self.endpoint}/{attribute_id}"

        utils.print_log(
            self._logger,
            "Calling delete attributes by managed system ID"
            f" and attribute ID endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)



"""Credentials module, all the logic to manage Credentials from Password Safe API"""

import logging

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject


class Credentials(APIObject):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/credentials")

        # Schema rules used for validations
        self._schema = {
            "request_id": {"type": "integer", "nullable": True},
            "alias_id": {"type": "integer", "nullable": True},
            "type": {
                "type": "string",
                "allowed": ["password", "dsskey"],
                "nullable": True,
            },
        }
        self._validator = self._validator = utils.Validator(self._schema)

    def get_credentials_by_request_id(self, request_id: int, type: str = None) -> dict:
        """
        Returns credentials by request ID.

        API: GET Credentials/{requestid}

        Args:
            request_id (int): The request ID.
            type (str, optional): The type of credentials to return. Options are:
                - password (default): Returns the password in the response body.
                - dsskey: Returns the DSS private key in the response body.
        Returns:
            dict: Credentials.
        """

        attributes = {"request_id": request_id, "type": type}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        params = {"type": type}

        query_string = self.make_query_string(params)

        endpoint = f"{self.endpoint}/{request_id}?{query_string}"

        utils.print_log(
            self._logger,
            f"Calling get_credentials_by_request_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_credentials_by_alias_id(
        self, alias_id: int, request_id: int, type: str = None
    ) -> dict:
        """
        Returns credentials by alias ID.

        API: GET Aliases/{aliasId}/Credentials/{requestId}

        Args:
            alias_id (int): The alias ID.
            request_id (int): The request ID.
            type (str, optional): The type of credentials to return. Options are:
                - password (default): Returns the password in the response body.
                - dsskey: Returns the DSS private key in the response body.
        Returns:
            dict: Credentials.
        """

        attributes = {"alias_id": alias_id, "request_id": request_id, "type": type}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        params = {"type": type}

        query_string = self.make_query_string(params)

        endpoint = f"/aliases/{alias_id}/credentials/{request_id}?{query_string}"

        utils.print_log(
            self._logger,
            f"Calling get_credentials_by_alias_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()



from secrets_safe_library.constants.versions import Version
from secrets_safe_library.validators.base import BaseValidator


class _ManagedSystemSchemaValidator:
    def __init__(self):
        self.common_schema = {
            "contact_email": {"type": "string", "maxlength": 1000, "nullable": False},
            "description": {"type": "string", "maxlength": 255, "nullable": True},
            "timeout": {"type": "integer", "min": 1, "max": 3600, "default": 30},
            "password_rule_id": {"type": "integer", "nullable": False, "default": 0},
            "release_duration": {
                "type": "integer",
                "min": 1,
                "max": 525600,
                "default": 120,
            },
            "max_release_duration": {
                "type": "integer",
                "min": 1,
                "max": 525600,
                "default": 525600,
            },
            "isa_release_duration": {
                "type": "integer",
                "min": 1,
                "max": 525600,
                "default": 120,
            },
            "auto_management_flag": {"type": "boolean", "default": False},
            "functional_account_id": {"type": "integer", "nullable": True},
            "check_password_flag": {"type": "boolean", "default": False},
            "change_password_after_any_release_flag": {
                "type": "boolean",
                "default": False,
            },
            "reset_password_on_mismatch_flag": {"type": "boolean", "default": False},
            "change_frequency_type": {
                "type": "string",
                "allowed": ["first", "last", "xdays"],
                "default": "first",
            },
            "change_frequency_days": {
                "type": "integer",
                "min": 1,
                "max": 999,
                "nullable": True,
            },
            "change_time": {
                "type": "string",
                "regex": r"^(?:[01]\d|2[0-3]):[0-5]\d$",  # 24-hour format (HH:MM)
                "default": "23:30",
            },
        }

    def get_asset_schema(self, version: str) -> dict:
        if version == Version.V3_0.value:
            return {
                **self.common_schema,
                "platform_id": {"type": "integer", "nullable": False},
                "port": {"type": "integer", "nullable": True},
                "ssh_key_enforcement_mode": {
                    "type": "integer",
                    "allowed": [0, 1, 2],
                    "nullable": True,
                    "default": 0,
                },
                "dss_key_rule_id": {"type": "integer", "nullable": True, "default": 0},
                "login_account_id": {"type": "integer", "nullable": True},
                "elevation_command": {
                    "type": "string",
                    "allowed": ["sudo", "pbrun", "pmrun"],
                    "nullable": True,
                },
            }
        elif version == Version.V3_1.value:
            schema = self.get_asset_schema(Version.V3_0.value)
            schema["remote_client_type"] = {
                "type": "string",
                "allowed": ["None", "EPM"],
                "nullable": False,
            }
            return schema
        elif version == Version.V3_2.value:
            schema = self.get_asset_schema(Version.V3_1.value)
            schema["application_host_id"] = {"type": "integer", "nullable": True}
            schema["is_application_host"] = {"type": "boolean", "default": False}
            return schema
        else:
            raise ValueError(f"Unsupported version: {version}")

    def get_database_schema(self) -> dict:
        return self.common_schema

    def get_workgroup_schema(self, version: str) -> dict:
        if version == Version.V3_0.value:
            return {
                **self.common_schema,
                "entity_type_id": {"type": "integer", "nullable": False},
                "host_name": {"type": "string", "maxlength": 128, "nullable": False},
                "ip_address": {"type": "string", "maxlength": 45, "nullable": False},
                "dns_name": {"type": "string", "maxlength": 255, "nullable": False},
                "instance_name": {"type": "string", "maxlength": 100, "nullable": True},
                "is_default_instance": {"type": "boolean", "nullable": True},
                "template": {"type": "string", "nullable": True},
                "forest_name": {"type": "string", "maxlength": 64, "nullable": True},
                "use_ssl": {"type": "boolean", "nullable": True, "default": False},
                "platform_id": {"type": "integer", "nullable": False},
                "net_bios_name": {"type": "string", "maxlength": 15, "nullable": True},
                "account_name_format": {
                    "type": "integer",
                    "allowed": [0, 1, 2],
                    "nullable": False,
                },
                "oracle_internet_directory_id": {
                    "type": "string",
                    "regex": r"^[0-9a-fA-F-]{36}$",
                    "nullable": True,
                },
                "oracle_internet_directory_service_name": {
                    "type": "string",
                    "maxlength": 200,
                    "nullable": True,
                },
                "elevation_command": {
                    "type": "string",
                    "allowed": ["sudo", "pbrun", "pmrun"],
                    "nullable": True,
                },
                "access_url": {"type": "string", "maxlength": 2048, "nullable": True},
                "ssh_key_enforcement_mode": {
                    "type": "integer",
                    "allowed": [0, 1, 2],
                    "nullable": True,
                    "default": 0,
                },
            }
        elif version == Version.V3_1.value:
            schema = self.get_workgroup_schema(Version.V3_0.value)
            schema["remote_client_type"] = {
                "type": "string",
                "allowed": ["None", "EPM"],
                "nullable": True,
            }
            return schema
        elif version == Version.V3_2.value:
            schema = self.get_workgroup_schema(Version.V3_1.value)
            schema["application_host_id"] = {"type": "integer", "nullable": True}
            schema["is_application_host"] = {"type": "boolean", "default": False}
            return schema
        elif version == Version.V3_3.value:
            return self.get_workgroup_schema(Version.V3_2.value)
        else:
            raise ValueError(f"Unsupported version: {version}")

    def get_update_schema(self, version: str) -> dict:
        return self.get_workgroup_schema(version)


class ManagedSystemValidator(BaseValidator):
    """Validator for managed systems operations."""

    def __init__(self):
        self.schema_validator = _ManagedSystemSchemaValidator()

    def get_schema(self, operation: str, version: str = Version.DEFAULT.value) -> dict:
        """
        Retrieve the schema for the specified operation and version.

        Args:
            operation (str): The operation type (e.g., 'create_by_asset',
                'create_by_database', 'create_by_workgroup').
            version (str): The version (e.g., '3.0', '3.1', '3.2', '3.3', 'DEFAULT')

        Returns:
            dict: The schema for the specified operation and version.
        """
        if operation == "create_by_asset":
            return self.schema_validator.get_asset_schema(version)

        if operation == "create_by_database":
            return self.schema_validator.get_database_schema()

        if operation == "create_by_workgroup":
            return self.schema_validator.get_workgroup_schema(version)

        if operation == "update":
            return self.schema_validator.get_update_schema(version)

        raise ValueError(f"Unsupported operation: {operation}")

    def validate(
        self,
        data: dict,
        operation: str,
        version: str = Version.DEFAULT.value,
        allow_unknown: bool = True,
        update: bool = False,
    ) -> dict:
        schema = self.get_schema(operation, version)
        data = super().validate(data, schema, allow_unknown, update)
        return data



from enum import Enum


class UserType(Enum):
    """
    Available user types.
    """

    USER_TYPE_BI = "BeyondInsight"
    USER_TYPE_AD = "ActiveDirectory"
    USER_TYPE_LDAP = "LdapDirectory"
    USER_TYPE_APP = "Application"
    USER_TYPE_ENTRAID = "EntraId"



"""ManagedSystems Module, all the logic to retrieve managed systems from PS API"""

import logging

from secrets_safe_library import utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import (
    POST_MANAGED_SYSTEMS_ASSETID,
    POST_MANAGED_SYSTEMS_DATABASEID,
    POST_MANAGED_SYSTEMS_WORKGROUPID,
    PUT_MANAGED_SYSTEMS_MANAGEDSYSTEMID,
)
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.managed_systems import fields as managed_system_fields
from secrets_safe_library.validators.managed_systems import ManagedSystemValidator


class ManagedSystem(APIObject, ManagedSystemValidator):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        # Initialize the APIObject parent class
        APIObject.__init__(self, authentication, logger)

        # Initialize the ManagedSystemValidator parent class
        ManagedSystemValidator.__init__(self)

    def get_managed_systems(
        self, limit: int = None, offset: int = None, type: int = None, name: str = None
    ) -> list:
        """
        Returns a list of managed systems.

        API: GET ManagedSystems/

        Args:
            limit (int, optional): Number of records to return. (default: 100000)
            offset (int, optional): Records to skip before returning results
                                    (use with limit). (default: 0)
            type (int, optional): The entity type of the managed system.
            name (str, optional): The managed system name.

        Returns:
            list: List of managed systems.
        """

        params = {"limit": limit, "offset": offset, "type": type, "name": name}
        query_string = self.make_query_string(params)
        endpoint = f"/managedsystems?{query_string}"

        utils.print_log(
            self._logger,
            f"Calling get_managed_systems endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_managed_system_by_id(self, managed_system_id: int) -> dict:
        """
        Find a managed system by ID.

        API: GET ManagedSystems/{id}

        Args:
            managed_system_id (int): The managed system ID.

        Returns:
            dict: Managed system object.
        """

        endpoint = f"/managedsystems/{managed_system_id}"

        utils.print_log(
            self._logger,
            f"Calling get_managed_system_by_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_managed_system_by_asset_id(self, asset_id: int) -> dict:
        """
        Find a managed system by asset ID.

        API: GET Assets/{assetId}/ManagedSystems

        Args:
            assetId (int): The asset ID.

        Returns:
            dict: Managed system object.
        """

        endpoint = f"/assets/{asset_id}/managedsystems"

        utils.print_log(
            self._logger,
            f"Calling get_managed_system_by_asset_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_managed_system_by_database_id(self, database_id: int) -> dict:
        """
        Find a managed system by database ID.

        API: GET Databases/{databaseID}/ManagedSystems

        Args:
            databaseID (int): The database ID.
        Returns:
            dict: Managed system object.
        """

        endpoint = f"/databases/{database_id}/managedsystems"

        utils.print_log(
            self._logger,
            f"Calling get_managed_system_by_database_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_managed_system_by_functional_account_id(
        self,
        functional_account_id: int,
        limit: int = None,
        offset: int = None,
        type: int = None,
        name: str = None,
    ) -> list:
        """
        Returns a list of managed systems auto-managed by the functional
        account referenced by ID.

        API: GET FunctionalAccounts/{id}/ManagedSystems

        Args:
            id (int): The functional account ID.
            limit (int, optional): Number of records to return. (default: 100000)
            offset (int, optional): Records to skip before returning results
                                    (use with limit). (default: 0)
            type (int, optional): The entity type of the managed system.
            name (str, optional): The managed system name.

        Returns:
            list: List of managed systems by functional account id.
        """
        params = {
            "limit": limit,
            "offset": offset,
            "type": type,
            "name": name,
        }
        query_string = self.make_query_string(params)
        endpoint = (
            f"/functionalaccounts/{functional_account_id}/managedsystems?"
            f"{query_string}"
        )

        utils.print_log(
            self._logger,
            f"Calling get_managed_system_by_functional_account_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def get_managed_system_by_workgroup_id(
        self, workgroup_id: int, limit: int = None, offset: int = None
    ) -> list:
        """
        Returns a list of managed systems by Workgroup ID.

        API: GET Workgroups/{id}/ManagedSystems

        Args:
            id (int): The workgroup ID.
            limit (int, optional): Number of records to return. (default: 100000)
            offset (int, optional): Records to skip before returning results
                                    (use with limit). (default: 0)

        Returns:
            list: List of managed systems by workgroup id.
        """

        params = {"limit": limit, "offset": offset}
        query_string = self.make_query_string(params)
        endpoint = f"/workgroups/{workgroup_id}/managedsystems?{query_string}"

        utils.print_log(
            self._logger,
            f"Calling get_managed_system_by_workgroup_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_managed_system_by_asset_id(
        self,
        *,
        asset_id: int,
        platform_id: int,
        contact_email: str,
        description: str,
        release_duration: int,
        max_release_duration: int,
        isa_release_duration: int,
        password_rule_id: int,
        check_password_flag: bool,
        change_password_after_any_release_flag: bool,
        reset_password_on_mismatch_flag: bool,
        change_frequency_type: str,
        change_frequency_days: int,
        change_time: str,
        is_application_host: bool,
        timeout: int = 30,
        port: int = None,
        ssh_key_enforcement_mode: int = None,
        dss_key_rule_id: int = None,
        login_account_id: int = None,
        auto_management_flag: bool = None,
        functional_account_id: int = None,
        elevation_command: str = None,
        remote_client_type: str = None,
        application_host_id: int = None,
    ) -> dict:
        """
        Creates a managed system for the asset referenced by ID.

        API: POST Assets/{assetId}/ManagedSystems

        Args:
            asset_id (int): The asset ID.
            platform_id (int): ID of the managed system platform..
            contact_email (str): Contact email (max 1000 characters).
            description (str): Decription (max 255 characters).
            timeout (int): Connection timeout. Length of time in seconds before a slow
                           or unresponsive connection to the system fails.
            release_duration (int):  Default release duration (default: 120).
            max_release_duration (int): Maximum release duration (default: 525600).
            isa_release_duration (int): ISA release duration (default: 120).
            password_rule_id (int): ID of the password rule (Default: 0).
            check_password_flag (bool): True to enable pswrd testing, otherwise false.
            change_password_after_any_release_flag(bool): True to change pswd on release
                                                      of a request, otherwise false.
            reset_password_on_mismatch_flag (bool): True to reset password on mismatch,
                                                otherwise false.
            change_frequency_type (str): The change frequency for scheduled password
                                       changes (default: first)
                            first: Changes scheduled for the first day of the month.
                            last: Changes scheduled for the last day of the month.
                            xdays: Changes scheduled every x days(change_frequency_days)
            change_frequency_days (int): if change_frequency_type is xdays, password
                                     changes take place this configured number of days.
            change_time (str): UTC time of day scheduled password changes take place.
                             (default: 23:30).
            is_application_host (bool): True if the managed system can be used as an
                                      application host, otherwise false (default:false)
                                      Cannot be set if application_host_id has a value.
            port (int, optional): The port used to connect to the host.
            ssh_key_enforcement_mode (int, optional): Enforcement mode for SSH host
                                                      keys.
                                        0: None.
                                        1: Auto. Auto accept initial key.
                                        2: Strict. Manually accept keys.
            dss_key_rule_id (int, optional): ID of the default DSS key rule assigned
                                to managed accounts created under this managed system.
            login_account_id (int, optional): ID of the functional account for
                                            SSH Session logins.
            auto_management_flag (bool, optional): Auto management flag
                                                   (default: False).
            functional_account_id (int, optional): ID functional account used for
                                       local managed account password changes.
            elevation_command (str, optional): Elevation command used to elevate
                                              privileges. (sudo, pbrun, pmrun)
            remote_client_type (str, optional): The remote client type (default: None).
                                        None: No remote client.
                                        EPM: Endpoint Privilege Management.
            application_host_id (int, optional): Managed system ID of the target
                                                application host (default: null).
                                                Must be an ID of a managed system
                                                whose is_application_host = true.

        Returns:
            dict: Managed system object.
        """
        attributes = {
            "platform_id": platform_id,
            "contact_email": contact_email,
            "description": description,
            "port": port,
            "timeout": timeout,
            "ssh_key_enforcement_mode": ssh_key_enforcement_mode,
            "password_rule_id": password_rule_id,
            "dss_key_rule_id": dss_key_rule_id,
            "login_account_id": login_account_id,
            "release_duration": release_duration,
            "max_release_duration": max_release_duration,
            "isa_release_duration": isa_release_duration,
            "auto_management_flag": auto_management_flag,
            "functional_account_id": functional_account_id,
            "elevation_command": elevation_command,
            "check_password_flag": check_password_flag,
            "change_password_after_any_release_flag": (
                change_password_after_any_release_flag
            ),
            "reset_password_on_mismatch_flag": reset_password_on_mismatch_flag,
            "change_frequency_type": change_frequency_type,
            "change_frequency_days": change_frequency_days,
            "change_time": change_time,
            "remote_client_type": remote_client_type,
            "application_host_id": application_host_id,
            "is_application_host": is_application_host,
        }

        validated_data = self.validate(
            attributes,
            operation="create_by_asset",
            version=self._authentication._api_version,
        )

        req_structure = self.get_request_body_version(
            managed_system_fields, POST_MANAGED_SYSTEMS_ASSETID
        )

        req_body = self.generate_request_body(
            req_structure,
            **validated_data,
        )

        endpoint = f"/assets/{asset_id}/managedsystems"

        utils.print_log(
            self._logger,
            f"Calling post_managed_system_by_asset_id endpoint: {endpoint}",
            logging.DEBUG,
        )

        response = self._run_post_request(
            endpoint, req_body, expected_status_code=[200, 201]
        )
        status_code = response.status_code

        return response.json(), status_code

    def post_managed_system_by_database_id(
        self,
        database_id: int,
        contact_email: str,
        description: str,
        timeout: int,
        password_rule_id: int,
        release_duration: int,
        max_release_duration: int,
        isa_release_duration: int,
        auto_management_flag: bool,
        check_password_flag: bool,
        change_password_after_any_release_flag: bool,
        reset_password_on_mismatch_flag: bool,
        change_frequency_type: str,
        change_frequency_days: int,
        change_time: str,
        functional_account_id: int = None,
    ) -> dict:
        """
        Creates a managed system for the database referenced by ID.

        API: POST Databases/{databaseID}/ManagedSystems

        Args:
            database_id (int): The database ID.
            contact_email (str): Contact email (max 1000 characters).
            description (str): Decription (max 255 characters).
            timeout (int): Connection timeout. Length of time in seconds before a slow
                           or unresponsive connection to the system fails.
            password_rule_id (int): ID of the password rule (Default: 0).
            release_duration (int):  Default release duration (default: 120).
            max_release_duration (int): Maximum release duration (default: 525600).
            isa_release_duration (int): ISA release duration (default: 120).
            auto_management_flag (bool): Auto management flag (default: False).
            check_password_flag (bool): True to enable pswrd testing, otherwise false.
            change_password_after_any_release_flag(bool): True to change pswd on release
                                                      of a request, otherwise false.
            reset_password_on_mismatch_flag (bool): True to reset password on mismatch,
                                                otherwise false.
            change_frequency_type (str): The change frequency for scheduled password
                                       changes (default: first)
                            first: Changes scheduled for the first day of the month.
                            last: Changes scheduled for the last day of the month.
                            xdays: Changes scheduled every x days(change_frequency_days)
            change_frequency_days (int): if change_frequency_type is xdays, password
                                     changes take place this configured number of days.
            change_time (str): UTC time of day scheduled password changes take place.
                             (default: 23:30).
            functional_account_id (int, optional): ID functional account used for local
                                        managed account password changes.

        Returns:
            dict: Managed system object.
        """

        attributes = {
            "contact_email": contact_email,
            "description": description,
            "timeout": timeout,
            "password_rule_id": password_rule_id,
            "release_duration": release_duration,
            "max_release_duration": max_release_duration,
            "isa_release_duration": isa_release_duration,
            "auto_management_flag": auto_management_flag,
            "functional_account_id": functional_account_id,
            "check_password_flag": check_password_flag,
            "change_password_after_any_release_flag": (
                change_password_after_any_release_flag
            ),
            "reset_password_on_mismatch_flag": reset_password_on_mismatch_flag,
            "change_frequency_type": change_frequency_type,
            "change_frequency_days": change_frequency_days,
            "change_time": change_time,
        }

        validated_data = self.validate(attributes, operation="create_by_database")

        req_structure = self.get_request_body_version(
            managed_system_fields, POST_MANAGED_SYSTEMS_DATABASEID, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            **validated_data,
        )

        endpoint = f"/databases/{database_id}/managedsystems"

        utils.print_log(
            self._logger,
            f"Calling post_managed_system_by_database_id endpoint: {endpoint}",
            logging.DEBUG,
        )

        response = self._run_post_request(
            endpoint, req_body, expected_status_code=[200, 201]
        )
        status_code = response.status_code

        return response.json(), status_code

    def post_managed_system_by_workgroup_id(
        self,
        workgroup_id: int,
        entity_type_id: int,
        host_name: str,
        ip_address: str,
        dns_name: str,
        instance_name: str,
        template: str,
        forest_name: str,
        platform_id: int,
        net_bios_name: str,
        contact_email: str,
        description: str,
        timeout: int,
        password_rule_id: int,
        account_name_format: int,
        oracle_internet_directory_service_name: str,
        release_duration: int,
        max_release_duration: int,
        isa_release_duration: int,
        auto_management_flag: bool,
        check_password_flag: bool,
        change_password_after_any_release_flag: bool,
        reset_password_on_mismatch_flag: bool,
        change_frequency_type: str,
        change_frequency_days: int,
        change_time: str,
        remote_client_type: str,
        is_application_host: bool,
        access_url: str,
        is_default_instance: bool = None,
        use_ssl: bool = None,
        port: int = None,
        ssh_key_enforcement_mode: int = None,
        dss_key_rule_id: int = None,
        login_account_id: int = None,
        oracle_internet_directory_id: str = None,
        functional_account_id: int = None,
        elevation_command: str = None,
        application_host_id: int = None,
    ) -> dict:
        """
        Creates a managed system for the workgroup referenced by ID.

        API: POST Workgroups/{workgroupID}/ManagedSystems

        Args:
            workgroup_id (int): The workgroup ID.
            entity_type_id (int): The entity type ID.
            host_name (str): The host name.
            ip_address (str): The IP address.
            dns_name (str): The DNS name.
            instance_name (str): The instance name.
            template (str): The template."
            forest_name (str): The forest name.
            platform_id (int): The platform ID.
            net_bios_name (str): The NetBIOS name.
            contact_email (str): The contact email.
            description (str): The description.
            timeout (int): The timeout.
            password_rule_id (int): The password rule ID.
            account_name_format (int): The account name format.
            oracle_internet_directory_service_name (str): The Oracle Internet Directory
                                                         service name.
            release_duration (int): The release duration.
            max_release_duration (int): The maximum release duration.
            isa_release_duration (int): The ISA release duration.
            auto_management_flag (bool): True if auto management is enabled.
            check_password_flag (bool): True if password checking is enabled.
            change_password_after_any_release_flag (bool): True if password change
                                                           after any release is enabled.
            reset_password_on_mismatch_flag (bool): True if password reset on
                                                    mismatch is enabled.
            change_frequency_type (str): The change frequency type.
            change_frequency_days (int): The change frequency days.
            change_time (str): The change time.
            remote_client_type (str): The remote client type.
            is_application_host (bool): True if the managed system is an
                                        application host.
            access_url (str): The access URL.
            is_default_instance (bool, optional): True if the instance is the
                                                default instance.
            use_ssl (bool, optional): True if SSL is used.
            port (int, optional): The port number.
            ssh_key_enforcement_mode (int, optional): The SSH key enforcement mode.
            dss_key_rule_id (int): The DSS key rule ID.
            login_account_id (int, optional): The login account ID.
            oracle_internet_directory_id (str, optional): The Oracle Internet
                                                          Directory GUID.
            functional_account_id (int, optional): The functional account ID.
            elevation_command (str, optional): The elevation command.
            application_host_id (int, optional): The application host ID.

        Returns:
            dict: Managed system object.
        """
        attributes = {
            "entity_type_id": entity_type_id,
            "host_name": host_name,
            "ip_address": ip_address,
            "dns_name": dns_name,
            "instance_name": instance_name,
            "is_default_instance": is_default_instance,
            "template": template,
            "forest_name": forest_name,
            "use_ssl": use_ssl,
            "platform_id": platform_id,
            "net_bios_name": net_bios_name,
            "contact_email": contact_email,
            "description": description,
            "port": port,
            "timeout": timeout,
            "ssh_key_enforcement_mode": ssh_key_enforcement_mode,
            "password_rule_id": password_rule_id,
            "dss_key_rule_id": dss_key_rule_id,
            "login_account_id": login_account_id,
            "account_name_format": account_name_format,
            "oracle_internet_directory_id": oracle_internet_directory_id,
            "oracle_internet_directory_service_name": (
                oracle_internet_directory_service_name
            ),
            "release_duration": release_duration,
            "max_release_duration": max_release_duration,
            "isa_release_duration": isa_release_duration,
            "auto_management_flag": auto_management_flag,
            "functional_account_id": functional_account_id,
            "elevation_command": elevation_command,
            "check_password_flag": check_password_flag,
            "change_password_after_any_release_flag": (
                change_password_after_any_release_flag
            ),
            "reset_password_on_mismatch_flag": reset_password_on_mismatch_flag,
            "change_frequency_type": change_frequency_type,
            "change_frequency_days": change_frequency_days,
            "change_time": change_time,
            "remote_client_type": remote_client_type,
            "application_host_id": application_host_id,
            "is_application_host": is_application_host,
            "access_url": access_url,
        }

        validated_data = self.validate(
            attributes,
            operation="create_by_workgroup",
            version=self._authentication._api_version,
        )

        req_structure = self.get_request_body_version(
            managed_system_fields, POST_MANAGED_SYSTEMS_WORKGROUPID
        )

        req_body = self.generate_request_body(
            req_structure,
            **validated_data,
        )

        endpoint = f"/workgroups/{workgroup_id}/managedsystems"

        utils.print_log(
            self._logger,
            f"Calling post_managed_system_by_workgroup_id endpoint: {endpoint}",
            logging.DEBUG,
        )

        response = self._run_post_request(
            endpoint, req_body, expected_status_code=[200, 201]
        )
        status_code = response.status_code

        return response.json(), status_code

    def delete_managed_system_by_id(self, managed_system_id: int) -> None:
        """
        Deletes a managed system by ID.

        API: DELETE ManagedSystems/{id}

        Args:
            managed_system_id (int): The managed system ID.

        Returns:
            None: If deletion is successful no exceptions.DeletionError is raised.
        """

        endpoint = f"/managedsystems/{managed_system_id}"

        utils.print_log(
            self._logger,
            f"Calling delete_managed_system_by_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        self._run_delete_request(endpoint)

    def put_managed_system_by_id(
        self,
        managed_system_id: int,
        workgroup_id: int,
        host_name: str,
        ip_address: str,
        dns_name: str,
        instance_name: str,
        template: str,
        forest_name: str,
        platform_id: int,
        net_bios_name: str,
        contact_email: str,
        description: str,
        timeout: int,
        password_rule_id: int,
        release_duration: int,
        max_release_duration: int,
        isa_release_duration: int,
        auto_management_flag: bool,
        check_password_flag: bool,
        change_password_after_any_release_flag: bool,
        reset_password_on_mismatch_flag: bool,
        change_frequency_type: str,
        change_frequency_days: int,
        change_time: str,
        remote_client_type: str,
        is_application_host: bool,
        access_url: str,
        is_default_instance: bool = None,
        use_ssl: bool = None,
        port: int = None,
        ssh_key_enforcement_mode: int = None,
        dss_key_rule_id: int = None,
        login_account_id: int = None,
        functional_account_id: int = None,
        elevation_command: str = None,
        application_host_id: int = None,
    ) -> dict:
        """
        Updates a managed system by ID.

        API: PUT ManagedSystems/{id}
        Args:
            managed_system_id (int): The managed system ID.
            workgroup_id (int): The workgroup ID.
            host_name (str): The host name.
            ip_address (str): The IP address.
            dns_name (str): The DNS name.
            instance_name (str): The instance name.
            template (str): The template.
            forest_name (str): The forest name.
            platform_id (int): The platform ID.
            net_bios_name (str): The NetBIOS name.
            contact_email (str): The contact email.
            description (str): The description.
            timeout (int): The timeout.
            password_rule_id (int): The password rule ID.
            release_duration (int): The release duration.
            max_release_duration (int): The maximum release duration.
            isa_release_duration (int): The ISA release duration.
            auto_management_flag (bool): True if auto management is enabled.
            check_password_flag (bool): True if password checking is enabled.
            change_password_after_any_release_flag (bool): True if password change
                                                           after any release is enabled.
            reset_password_on_mismatch_flag (bool): True if password reset on
                                                    mismatch is enabled.
            change_frequency_type (str): The change frequency type.
            change_frequency_days (int): The change frequency days.
            change_time (str): The change time.
            remote_client_type (str): The remote client type.
            is_application_host (bool): True if the managed system is an
                                        application host.
            access_url (str): The access URL.
            is_default_instance (bool, optional): True if the instance is the
                                                default instance.
            use_ssl (bool, optional): True if SSL is used.
            port (int, optional): The port number.
            ssh_key_enforcement_mode (int, optional): The SSH key enforcement mode.
            dss_key_rule_id (int, optional): The DSS key rule ID.
            login_account_id (int, optional): The login account ID.
            functional_account_id (int, optional): The functional account ID.
            elevation_command (str, optional): The elevation command.
            application_host_id (int, optional): The application host ID.

        Returns:
            dict: Managed system object.
        """
        attributes = {
            "workgroup_id": workgroup_id,
            "host_name": host_name,
            "ip_address": ip_address,
            "dns_name": dns_name,
            "instance_name": instance_name,
            "template": template,
            "forest_name": forest_name,
            "platform_id": platform_id,
            "net_bios_name": net_bios_name,
            "contact_email": contact_email,
            "description": description,
            "timeout": timeout,
            "password_rule_id": password_rule_id,
            "release_duration": release_duration,
            "max_release_duration": max_release_duration,
            "isa_release_duration": isa_release_duration,
            "auto_management_flag": auto_management_flag,
            "check_password_flag": check_password_flag,
            "change_password_after_any_release_flag": (
                change_password_after_any_release_flag
            ),
            "reset_password_on_mismatch_flag": reset_password_on_mismatch_flag,
            "change_frequency_type": change_frequency_type,
            "change_frequency_days": change_frequency_days,
            "change_time": change_time,
            "remote_client_type": remote_client_type,
            "is_application_host": is_application_host,
            "access_url": access_url,
            "is_default_instance": is_default_instance,
            "use_ssl": use_ssl,
            "port": port,
            "ssh_key_enforcement_mode": ssh_key_enforcement_mode,
            "dss_key_rule_id": dss_key_rule_id,
            "login_account_id": login_account_id,
            "functional_account_id": functional_account_id,
            "elevation_command": elevation_command,
            "application_host_id": application_host_id,
        }

        validated_data = self.validate(
            attributes,
            operation="update",
            version=self._authentication._api_version,
            update=True,
        )

        req_structure = self.get_request_body_version(
            managed_system_fields, PUT_MANAGED_SYSTEMS_MANAGEDSYSTEMID
        )

        req_body = self.generate_request_body(
            req_structure,
            **validated_data,
        )

        endpoint = f"/managedsystems/{managed_system_id}"

        utils.print_log(
            self._logger,
            f"Calling put_managed_system_by_id endpoint: {endpoint}",
            logging.DEBUG,
        )

        response = self._run_put_request(endpoint, req_body)

        return response.json()



"""
This file contains the fields mapping for each endpoint and version related to
Sessions.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-api
"""

from secrets_safe_library.constants.endpoints import (
    GET_SESSIONS,
    GET_SESSIONS_ID,
    POST_SESSIONS_ADMIN,
    POST_SESSIONS_REQUEST_ID,
)
from secrets_safe_library.constants.versions import Version

fields = {
    GET_SESSIONS: {
        Version.DEFAULT: [
            "SessionID",
            "UserID",
            "NodeID",
            "Status",
            "ArchiveStatus",
            "Protocol",
            "StartTime",
            "EndTime",
            "Duration",
            "AssetName",
            "ManagedSystemID",
            "ManagedAccountID",
            "ManagedAccountName",
            "RecordKey",
            "Token",
            "ApplicationID",
            "RequestID",
            "SessionType",
        ],
    },
    GET_SESSIONS_ID: {
        Version.DEFAULT: [
            "SessionID",
            "UserID",
            "NodeID",
            "Status",
            "ArchiveStatus",
            "Protocol",
            "StartTime",
            "EndTime",
            "Duration",
            "AssetName",
            "ManagedSystemID",
            "ManagedAccountID",
            "ManagedAccountName",
            "RecordKey",
            "Token",
            "ApplicationID",
            "RequestID",
            "SessionType",
        ],
    },
    POST_SESSIONS_REQUEST_ID: {
        Version.DEFAULT: {
            "SessionType": str,
            "NodeID": str,
        },
    },
    POST_SESSIONS_ADMIN: {
        Version.DEFAULT: {
            "SessionType": str,
            "HostName": str,
            "Port": int,
            "DomainName": str,
            "UserName": str,
            "Password": str,
            "Reason": str,
            "Resolution": str,
            "RDPAdminSwitch": bool,
            "SmartSizing": bool,
            "NodeID": str,
            "Record": bool,
        },
    },
}



"""Safes module, all the logic to manage safes from PS API"""

import logging
from typing import Tuple

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.core import APIObject
from secrets_safe_library.mixins import DeleteByIdMixin, GetByIdMixin, ListMixin


class Safe(APIObject, GetByIdMixin, DeleteByIdMixin, ListMixin):

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/secrets-safe/safes")

        # Schema rules used for validations
        self._schema = {
            "name": {"type": "string", "maxlength": 256, "nullable": True},
            "description": {"type": "string", "maxlength": 256, "nullable": True},
        }
        self._validator = Validator(self._schema)

    def create_safe(
        self,
        name: str,
        description: str = "",
    ) -> dict:
        """
        Creates a new Safe.

        API: POST Secrets-Safe/Safes/

        Args:
            name (str): The safe name.
            description (str, optional): The safe description.

        Returns:
            dict: Created Safe object.
        """

        attributes = {"name": name, "description": description}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Name": name,
            "Description": description,
        }
        endpoint = "/secrets-safe/safes"

        utils.print_log(
            self._logger, f"Calling create safe endpoint: {endpoint}", logging.DEBUG
        )
        response = self._run_post_request(endpoint, payload, include_api_version=False)

        return response.json()

    def update_safe(
        self,
        safe_id: str,
        name: str,
        description: str = "",
    ) -> Tuple[str, int]:
        """
        Update an existing Safe using its ID.

        API: PUT secrets-safe/safes/{id}

        Args:
            safe_id (str): The safe ID (GUID).
            name (str): The safe name.
            description (str, optional): The safe description.

        Returns:
            Tuple[str, int]: Tuple containing raw response and int response status code.
        """

        attributes = {"name": name, "description": description}

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "Name": name,
            "Description": description,
        }
        endpoint = f"/secrets-safe/safes/{safe_id}"

        utils.print_log(
            self._logger, f"Calling update safe endpoint: {endpoint}", logging.DEBUG
        )
        response = self._run_put_request(
            endpoint,
            payload,
            include_api_version=False,
            expected_status_code=[204, 409],
        )

        return response.text, response.status_code



"""
This file contains the fields mapping for each endpoint and version related to
Applications.

More info for the fields included in each endpoint and version:
- https://docs.beyondtrust.com/bips/docs/passwordsafe-apis
"""

from secrets_safe_library.constants.endpoints import (
    GET_APPLICATIONS,
    GET_APPLICATIONS_ID,
)
from secrets_safe_library.constants.versions import Version

DEFAULT_FIELDS = [
    "ApplicationID",
    "Name",
    "DisplayName",
    "Version",
    "Command",
    "Parameters",
    "Publisher",
    "ApplicationType",
    "FunctionalAccountID",
    "ManagedSystemID",
    "IsActive",
    "SmartRuleID",
]

fields = {
    GET_APPLICATIONS: {
        Version.DEFAULT: DEFAULT_FIELDS,
    },
    GET_APPLICATIONS_ID: {Version.DEFAULT: DEFAULT_FIELDS},
}



from dataclasses import dataclass


@dataclass
class SSHConfig:
    """Data class to hold SSH configuration details."""

    private_key: str = ""
    passphrase: str = ""
    elevation_command: str = ""



"""Sessions Module, all the logic to retrieve sessions from PS API"""

import logging

from cerberus import Validator

from secrets_safe_library import exceptions, utils
from secrets_safe_library.authentication import Authentication
from secrets_safe_library.constants.endpoints import POST_SESSIONS_ADMIN
from secrets_safe_library.constants.versions import Version
from secrets_safe_library.core import APIObject
from secrets_safe_library.mapping.sessions import fields as sessions_fields
from secrets_safe_library.mixins import GetByIdMixin


class Session(APIObject, GetByIdMixin):
    """
    Class to interact with sessions in PS API.
    """

    def __init__(self, authentication: Authentication, logger: logging.Logger = None):
        super().__init__(authentication, logger, endpoint="/sessions")

        # Schema rules used for validations
        self._schema = {
            "session_type": {"type": "string", "maxlength": 32, "nullable": True},
            "node_id": {"type": "integer", "nullable": True},
            "request_id": {"type": "integer", "nullable": True},
            "host_name": {"type": "string", "maxlength": 128, "nullable": True},
            "domain_name": {"type": "string", "maxlength": 50, "nullable": True},
            "user_name": {"type": "string", "maxlength": 200, "nullable": True},
        }
        self._validator = Validator(self._schema)

    def get_sessions(self, status: int = None, user_id: int = None) -> list:
        """
        Returns a list of sessions.

        API: GET Sessions

        Args:
            status (int, optional): A single value or comma-delimited list of values:
                - 0: Not Started
                - 1: In Progress
                - 2: Completed
                - 5: Locked
                - 7: Terminated (deprecated)
                - 8: Logged Off
                - 9: Disconnected (RDP only)
            user_id (int, optional): ID of the user that requested the session.

        Returns:
            list: List of sessions.
        """
        params = {"status": status, "userID": user_id}
        query_string = self.make_query_string(params)
        endpoint = f"{self.endpoint}?{query_string}"

        utils.print_log(
            self._logger,
            f"Calling get_sessions endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_get_request(endpoint, include_api_version=False)

        return response.json()

    def post_sessions_request_id(
        self, request_id: int, session_type: str, node_id: int = None
    ) -> dict:
        """
        Posts a session request by ID.

        API: POST Sessions RequestID

        Args:
            request_id (int): The ID of the session request.
            session_type (str): The type of session to create.
                Options: SSH or sshticket, RDP or rdpticket, rdpfile, app, or appfile.
            node_id (int, optional):  ID of the node that should be used to establish
                    the session. If NodeID is not given or if the Remote Session Proxy
                    feature is disabled, uses the local node.

        Returns:
            dict: The response from the API.
        """

        attributes = {
            "request_id": request_id,
            "session_type": session_type,
            "node_id": node_id,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        payload = {
            "SessionType": session_type,
            "NodeID": node_id,
        }

        endpoint = f"/requests/{request_id}/sessions"
        utils.print_log(
            self._logger,
            f"Calling post_sessions_request_id endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, payload, include_api_version=False)

        return response.json()

    def post_sessions_admin(
        self,
        session_type: str,
        host_name: str,
        user_name: str,
        password: str,
        port: int = None,
        domain_name: str = None,
        reason: str = None,
        resolution: str = None,
        rdp_admin_switch: bool = None,
        smart_sizing: bool = None,
        node_id: int = None,
        record: bool = None,
    ) -> dict:
        """
        Posts a session request as an admin.

        API: POST Sessions Admin

        Args:
            session_type (str): The type of session to create.
                Options: SSH or sshticket, RDP or rdpticket, rdpfile, app, or appfile.
            host_name (str): The hostname or IP address of the target system.
            user_name (str): The username for the session.
            port (int, optional): The port number for the session. Defaults to None.
            domain_name (str, optional): The domain name for session. Defaults to None.
            password (str): The password for the session.
            reason (str, optional): Reason for the session request. Defaults to None.
            resolution (str, optional): Resolution for the session. Defaults to None.
            rdp_admin_switch (bool, optional): Whether to use RDP admin switch.
                                               Defaults None.
            smart_sizing (bool, optional): Whether to enable smart sizing.
                                            Defaults None.
            node_id (int, optional): ID of the node that should be used to establish
                    the session. If NodeID is not given or if the Remote Session Proxy
                    feature is disabled, uses the local node.
            record (bool, optional): Whether to record the session. Defaults to None.

        Returns:
            dict: The response from the API.
        """

        attributes = {
            "session_type": session_type,
            "node_id": node_id,
            "host_name": host_name,
            "domain_name": domain_name,
            "user_name": user_name,
        }

        if not self._validator.validate(attributes, update=True):
            raise exceptions.OptionsError(f"Please check: {self._validator.errors}")

        req_structure = self.get_request_body_version(
            sessions_fields, POST_SESSIONS_ADMIN, Version.DEFAULT
        )

        req_body = self.generate_request_body(
            req_structure,
            session_type=session_type,
            host_name=host_name,
            domain_name=domain_name,
            user_name=user_name,
            password=password,
            port=port,
            reason=reason,
            resolution=resolution,
            rdp_admin_switch=rdp_admin_switch,
            smart_sizing=smart_sizing,
            node_id=node_id,
            record=record,
        )

        endpoint = "/sessions/admin"
        utils.print_log(
            self._logger,
            f"Calling post_sessions_admin endpoint: {endpoint}",
            logging.DEBUG,
        )
        response = self._run_post_request(endpoint, req_body, include_api_version=False)

        return response.json()



from secrets_safe_library.validators.base import BaseValidator


class _ManagedAccountValidator:

    def get_list_schema(self) -> dict:
        return {
            "system_name": {"type": "string", "maxlength": 129, "nullable": True},
            "account_name": {"type": "string", "maxlength": 246, "nullable": True},
            "system_id": {"type": "integer", "nullable": True},
            "workgroup_name": {"type": "string", "maxlength": 256, "nullable": True},
            "application_display_name": {
                "type": "string",
                "maxlength": 256,
                "nullable": True,
            },
            "ip_address": {"type": "string", "maxlength": 45, "nullable": True},
            "type": {
                "type": "string",
                "allowed": [
                    "system",
                    "recent",
                    "domainlinked",
                    "database",
                    "cloud",
                    "application",
                ],
                "nullable": True,
            },
            "limit": {"type": "integer", "min": 1, "max": 1000, "default": 1000},
            "offset": {"type": "integer", "min": 0, "default": 0},
        }

    def get_create_schema(self) -> dict:
        """
        Returns the schema for creating a managed account.
        """
        return {
            "system_id": {"type": "integer", "nullable": False},
            "account_name": {"type": "string", "maxlength": 245, "nullable": False},
            "password": {
                "type": "string",
                "maxlength": 1000,
                "nullable": True,
                "is_required_if": {
                    "field": "auto_management_flag",
                    "value": False,
                },
            },
            "domain_name": {"type": "string", "maxlength": 50, "nullable": True},
            "user_principal_name": {
                "type": "string",
                "maxlength": 500,
                "nullable": True,
            },
            "sam_account_name": {"type": "string", "maxlength": 20, "nullable": True},
            "distinguished_name": {
                "type": "string",
                "maxlength": 1000,
                "nullable": True,
            },
            "private_key": {"type": "string", "nullable": True},
            "passphrase": {"type": "string", "nullable": True},
            "password_fallback_flag": {"type": "boolean", "default": False},
            "login_account_flag": {"type": "boolean", "default": False},
            "description": {"type": "string", "maxlength": 1024, "nullable": True},
            "password_rule_id": {"type": "integer", "default": 0},
            "api_enabled": {"type": "boolean", "default": False},
            "release_notification_email": {
                "type": "string",
                "maxlength": 255,
                "nullable": True,
            },
            "change_services_flag": {"type": "boolean", "default": False},
            "restart_services_flag": {"type": "boolean", "default": False},
            "change_tasks_flag": {"type": "boolean", "default": False},
            "release_duration": {
                "type": "integer",
                "min": 1,
                "max": 525600,
                "default": 120,
            },
            "max_release_duration": {
                "type": "integer",
                "min": 1,
                "max": 525600,
                "default": 525600,
            },
            "isa_release_duration": {
                "type": "integer",
                "min": 1,
                "max": 525600,
                "default": 120,
            },
            "max_concurrent_requests": {
                "type": "integer",
                "min": 0,
                "max": 999,
                "default": 1,
            },
            "auto_management_flag": {"type": "boolean", "default": False},
            "dss_auto_management_flag": {"type": "boolean", "default": False},
            "check_password_flag": {"type": "boolean", "default": False},
            "reset_password_on_mismatch_flag": {"type": "boolean", "default": False},
            "change_password_after_any_release_flag": {
                "type": "boolean",
                "default": False,
            },
            "change_frequency_type": {
                "type": "string",
                "allowed": ["first", "last", "xdays"],
                "default": "first",
            },
            "change_frequency_days": {
                "type": "integer",
                "min": 1,
                "max": 999,
                "nullable": True,
                "is_required_if": {
                    "field": "change_frequency_type",
                    "value": "xdays",
                },
            },
            "change_time": {
                "type": "string",
                "regex": r"^(0[0-9]|1[0-9]|2[0-3]):[0-5][0-9]$",
                "default": "23:30",
            },
            "next_change_date": {
                "type": "string",
                "regex": r"^\d{4}-\d{2}-\d{2}$",
                "nullable": True,
            },
            "use_own_credentials": {"type": "boolean", "default": False},
            "change_iis_app_pool_flag": {"type": "boolean", "default": False},
            "restart_iis_app_pool_flag": {"type": "boolean", "default": False},
            "workgroup_id": {"type": "integer", "nullable": True},
            "change_windows_auto_logon_flag": {"type": "boolean", "default": False},
            "change_com_plus_flag": {"type": "boolean", "default": False},
            "change_dcom_flag": {"type": "boolean", "default": False},
            "change_scom_flag": {"type": "boolean", "default": False},
            "object_id": {"type": "string", "maxlength": 36, "nullable": True},
        }


class ManagedAccountValidator(BaseValidator):
    """Validator for managed accounts operations."""

    def __init__(self):
        self.schema_validator = _ManagedAccountValidator()

    def get_schema(self, operation: str) -> dict:
        """
        Retrieve the schema for the specified operation.

        Args:
            operation (str): The operation type (e.g., 'list', 'create').

        Returns:
            dict: The schema for the specified operation and version.
        """
        if operation == "list":
            return self.schema_validator.get_list_schema()

        if operation == "create":
            return self.schema_validator.get_create_schema()

        raise ValueError(f"Unsupported operation: {operation}")

    def validate(
        self,
        data: dict,
        operation: str,
        allow_unknown: bool = True,
        update: bool = False,
    ) -> dict:
        schema = self.get_schema(operation)
        data = super().validate(data, schema, allow_unknown, update)
        return data
